#!/usr/bin/perl -w
use strict;

use lib 'lib';
use lib 'inc';

use ExtUtils::MakeMaker;
use Config '%Config';
use File::Basename 	qw[dirname basename];
use File::Path;
use File::Spec;
use Cwd             qw[cwd];

my $package = 'Alien::Libpng';
(my $filename = $package) =~ s!::!/!g;
$filename .= ".pm";
my $lib;
my $cwd;
my $progs;

use vars qw[$DRY_RUN];
$DRY_RUN = 0;

my @Makefile_ARGS = (
    NAME            => $package,
    VERSION_FROM    => "lib/$filename",
    dist            => { COMPRESS => 'gzip -9f', SUFFIX => 'gz' },
    AUTHOR          => 'Max Maischein <corion@cpan.org>',
	ABSTRACT        => 'Supply external libpng library',
	PREREQ_PM       => { # These are the prerequisites for Alien::Util
	                     'IPC::Cmd' => 0,
                         'File::Fetch' => 0,
                         # These are the real prerequisites
                         'Alien::Zlib' => 0,
                       },
);

# First, manually check for our prerequisites and die if they aren't met:
WriteMakefile(@Makefile_ARGS);

### if you only want to write the makefile for 'make dist'
### invoke it as 'perl Makefile.PL dist'

unless( 'dist' eq ($ARGV[0] ||"" ) ) {

    eval q[
        use IPC::Cmd;
        use File::Fetch;
    ];
    if ($@) {
        warn <<PREREQUISITES;

IPC::Cmd and File::Fetch are not installed.

You will only be able to do

    make dist

PREREQUISITES

        exit
    };

    require $filename;

    $lib = $package->archive;

    # Seems like we have all we need
    require Alien::Util;
    Alien::Util->import();

    my @progs = qw[tar gzip make cc];
    if ($^O ne 'MSWin32' and $^O !~ /VMS/) {
      push @progs, 'sh'; # for ./configure
    };
    $progs = Alien::Util::find_progs(@progs);

    $lib = 'libpng-' . $package->lib_version . '-config';
    $cwd = cwd();
    chdir $cwd;

    ### fetch the sources ###
    my $src = Alien::Util::fetch($package->url);
    print "Succesfully fetched $lib\n";
    chdir $cwd;

    ### extract them ###
    Alien::Util::decompress($progs, $lib, $src );
    chdir $cwd;

    ### patch && install ###
    install_lib();
    chdir $cwd;
}

### Write the makefile, again, this time with all the correct object files ###
WriteMakefile(@Makefile_ARGS);

### SUB ROUTINES ###

sub install_lib {
    my $dirname  = File::Spec->rel2abs(dirname($0), 'src');
    my $lib_dir = File::Spec->catdir( $dirname, $lib );
    my $hints 	 = File::Spec->catdir( $dirname, 'hints', $^O );
    my $makefile = File::Spec->catdir( $lib_dir, 'Makefile' );
    my $prefix   = $package->prefix;
    my $include  = $package->include;
    my $lib      = $package->lib;
    
    eval q{use Alien::Zlib;};

    ### go to the zlib dir ###
    unless( chdir $lib_dir ) {
        die qq[Could not chdir to '$lib_dir' ($!) -- can not compile sources\n];
    }

    my @configure;
    my @install;

    if ($^O eq 'MSWin32') {
      push @configure, Alien::Util::copy_file('scripts/Makefile.vcawin32' => $makefile);

      my $perl = Alien::Util::add_quotes $^X;
      $perl =~ s!\\!\\\\!g;
      my $inc =   Alien::Util::add_quotes("-I" . quotemeta Alien::Zlib->include);
      my $lib = join " ", map { Alien::Util::add_quotes(quotemeta $_) } Alien::Zlib->lib_files;
      ### patch invocation for '$(RM)' to use Perl ###
      push @configure, Alien::Util::modify( 's/-I[.][.]\\\\zlib/'.$inc.'/', $makefile );
      push @configure, Alien::Util::modify( 's/-I[.][.]\\\\zlib\\\\zlib.lib/'.$lib.'/', $makefile );
      push @configure, Alien::Util::modify( sprintf('s/(RM\s*=\s*)\s*del/$1%s -MExtUtils::Command -e rm_f/', $perl), $makefile );      
    } elsif ($^O =~ /VMS/) {
      # This is fudged, I'm not sure what is the distinction
      # between VMS MMS and VMS-non-MMS
      Alien::Util::copy_file(File::Spec->catdir($lib_dir,'Makefile.vms') => File::Spec->catdir($lib_dir,$makefile));
      Alien::Util::copy_file(File::Spec->catdir($lib_dir,'jconfig.vms')  => File::Spec->catdir($lib_dir,'jconfig.h'));
    } else {
      @configure = ( [$progs->{'sh'}, 	'configure', @ARGV],

      # This is untested cargo-culted code:
      ### patch prefix ###
                     Alien::Util::modify( 's|(^prefix\s*=\s*).*$|$1'.$prefix.'|', $makefile),

      ### patch include dir ###
                     Alien::Util::modify( 's|(^includedir\s*=\s*).*$|$1'.$include.'|', $makefile),

      ### patch lib dir ###
                     Alien::Util::modify( 's|(^libdir\s*=\s*).*$|$1'.$lib.'|', $makefile),
      );
      push @install, [ $progs->{'make'},	'test'		],
      push @install, [ $progs->{'make'}, 	'install' ],
    };

    if (glob "src/*.$Config{_o}") {
      unshift @configure, [$progs->{'make'}, 'clean' ];
    };

    ### build the zlib sources, make sure to copy them into the lib/ dir ###
    my @cmds = (
        # Patch in CCFLAGS from %Config
        @configure,

        [$progs->{'make'},				],

        @install,
    );

    Alien::Util::process(@cmds);
    print "Succesfully compiled $lib\n";

    # Now, copy all we need into our lib directory
    for my $dir ($package->lib, $package->include) {
      mkpath $dir
      #  or die "Couldn't create '$dir': $!";
    };
    for ($package->lib_files) {
        Alien::Util::copy_file(File::Spec->catfile($lib_dir,basename($_)) => $_)->();
    };
    for ($package->include_files) {
        Alien::Util::copy_file(File::Spec->catfile($lib_dir,basename($_)) => $_)->();
    };
}
